const fs = require('fs')
const path = require('path')
const ncp = require('copy-paste')

const viewsFolder = 'src/views'
const ignoredFiles = ['search_list.html', 'search_tips.html', '404.html']
const ignoredFolders = ['public']
const baseUrl = process.argv[2] ? process.argv[2] : ''
const normalizedUrl = baseUrl.endsWith('/') ? baseUrl : `${baseUrl}/`

// 遍历指定目录下的所有 HTML 文件，并返回包含标题和 URL 的对象
function getHtmlFileInfo (folder, filename) {
    const filePath = path.join(viewsFolder, folder, filename)
    const fileContent = fs.readFileSync(filePath, 'utf8')
    const titleMatches = fileContent.match(/"title"\s*:\s*"([^\"]+)"/)
    const title = titleMatches ? titleMatches[1] + ':' : ''
    const url = `${normalizedUrl}${folder}/${filename}`
    return { title, url }
}

const htmlFiles = [
    {
        title: '首页:',
        url: normalizedUrl
    }
].concat(
    fs
        .readdirSync(viewsFolder)
        .filter((folder) => !ignoredFolders.includes(folder))
        .flatMap((folder) => {
            const folderPath = path.join(viewsFolder, folder)
            if (!fs.statSync(folderPath).isDirectory()) {
                return []
            }
            return fs
                .readdirSync(folderPath)
                .filter(
                    (filename) => filename.endsWith('.html') && !ignoredFiles.includes(filename)
                )
                .map((filename) => getHtmlFileInfo(folder, filename))
        })
)

// 将 HTML 文件信息数组转换成 JavaScript 代码字符串
const jsCode = `let nodeAry = ${JSON.stringify(htmlFiles)};
nodeAry.forEach((item) => {
    let p = document.createElement('p'),
        title = document.createElement('p'),
        a = document.createElement('a'),
        strNode = document.createTextNode(item.url),
        titleNode = document.createTextNode(item.title);
    a.setAttribute('href', item.url)
    a.append(strNode)
    title.append(titleNode)
    p.append(title)
    p.append(a)
    let editNode = $(window.frames["ueditor_0"])[0].contentDocument.body
    editNode.appendChild(p)
})
`

// 将 JavaScript 代码字符串复制到剪贴板
ncp.copy(jsCode, function () {
    console.log('\x1b[32m%s\x1b[0m', '√已复制代码到剪贴板')
})
